﻿#include <cmath>
#include "FractureNetwork.h"
#include <iostream>
#include <cstdlib>
#include <vector>
#include <ctime>
#include <fstream>
#include <iomanip>
#include <string>
#include <sstream>
#include <algorithm> 
#include "myrandom.h"
#include <map>
using namespace std;


void  FractalNetwork::Calculate_Fractal(FractalNetwork mynetwork)
{
	// inputs: text file with all coordinates of fault segments (x1,y1,x2,y2). 
	// outputs: 
	// name_circle_radius.txt:A text file to record positions of all circles used to cover the fault segments (name) for a given radius;
	// radius_name.txt: A text file to record the radii used to measure the set of fault segments (name);
	// Numcircles_name: A text file to record the number of circles to cover the set of fault segments (name) for each given radius of the circle;
	// name_changed: A text file to record positions of all fracture segments after transformation and it is used for plotting in Matlab;
	L = 10;// all the line segments should be preprocessed to be adapted to the 10*10 domain
	int capacity = 10000;
	int bs = 1;
	double *numCircle; numCircle = new double[1000];
	mynetwork.AssignCapacity(capacity, L, bs);
	// radius of the circle to cover the segments;
	double r[] = { 0.0001,0.0002,0.0003,0.0004,0.0005,0.0006,0.0008,0.0009,0.001,0.0015, 0.002,0.003,0.004,0.005,0.0063,0.007,0.008,0.009,0.01,0.012,0.015,0.018,0.02,0.024 , 0.028,  0.035,0.04,0.05,0.06,0.07,0.08,0.1,0.12,0.12,0.14,0.16,0.19,0.218,0.23,0.25,0.28,0.32,0.4,0.5,0.6,0.8,0.9,1,2,3,4,4.5,4.8 };
	int itern = (sizeof(r) / sizeof(*r));
	//cout << itern << endl;


	// For real fault maps, we should activate the rotation code. Line 16-17 in function "Calculate_compass_fractal"
	//	double largestangle =   CalAngle(Fractures[largestlengthindex]);
	//  transformFractures(FaultNum, largestangle, xmax, xmin, ymax, ymin);
	//  We provide three real fault maps, RealFault1.txt, RealFault2.txt, RealFault3.txt and three generated fault maps, Generated_1.txt, Generated_2.txt,Generated_3.txt;


	stringstream  filename;

	string faultname;
 
	Inputstring(faultname);
 
	//system("pause");
	filename << faultname;
	for (int i = 0; i < itern; i++)
	{
		numCircle[i] = mynetwork.Calculate_compass_fractal(filename.str(), r[i], r[itern - 1], i);// record the number of circles with a given radius;
		cout << "Radius: " << r[i] << ", Number of circle is  " << numCircle[i] << endl;
		cout << "*******************************************" << endl;

	}
	// output result: the number of circles and the radii;
	filename.str("");
	filename << "Data/Results/radius_" << faultname << ".txt";
	Outputarray(filename.str(), itern, r);
	filename.str("");
	filename << "Data/Results/Numcircles_" << faultname << ".txt";
	Outputarray(filename.str(), itern, numCircle);
}

//********************************  Fractal Dimension Calculation Functions ************************************

double FractalNetwork::Calculate_compass_fractal(string name, double r, double rmax, int iteration)
{
	stringstream  filename;
	double circlenum;
	double Lastcircle = 0;
	double xmin = NN; double ymax = -NN; double xmax = -NN; double ymin = NN;
	int Nblock;
	linesegment temp;
	pointcor intersect1, intersect2, p1, p2;
	pointcor tempcenter;
	int largestlengthindex;
	// input fractures and find the boundary of the domain;
	double totallength = 0;
	int FaultNum = InputFracture(name, xmax, xmin, ymax, ymin, largestlengthindex, totallength);
	// if we want to use the largest segment's angle as the rotation angle, we can active these two lines.
	// this is for real fault maps
	//Initilization
	double largestangle = CalAngle(Fractures[largestlengthindex]);
	transformFractures(FaultNum, largestangle, xmax, xmin, ymax, ymin);
	/*********************************************************************/
	//cout << "fault num is " << FaultNum << endl;
	double dx = xmax - xmin;
	double dy = ymax - ymin;
	// find the boundary size
	L = ceil(dy);
	if (dx > dy) L = ceil(dx);
	// if the system size can't be divided by the block size, we should augment the size.
	// However, in this program, all the fault segments are preprocessed, to make sure the L should be smaller than 10;
	if (L%bs != 0)
	{
		L = (L / bs + 1)*bs;
	}

	////////////////////////////////////////////
	for (int i = 0; i < FaultNum; i++)
	{
		Translate2D(Fractures[i], -xmin, -ymin);// translate fractures to (0,0) point;
	}

	xmin = 0; ymin = 0;
	xmax = dx; ymax = dy;
	sort(Fractures, Fractures + FaultNum, linecompare);// sort all fractures based on the y coordinates.
													   // for real fault maps with preprocessing, this part will not change anything.
	filename << "Data/Results/" << name << "_changed.txt";
	OutputFracture(filename.str(), FaultNum, Fractures);

	// find the blocks that each fault segment occupies;
	int n;
	n = Generatexycord(L, bs, xcord, ycord, 0, 0);
	/////////////////////////////////////
	for (int i = 0; i < FaultNum; i++)
	{
		// Nblock should always resets to 0 before calculate the blocks.
		Nblock = 0;
		Fracture = Fractures[i];
		BlockIndex(Allblocknumber, i, Nblock, n);
		tempFracturesindex[i] = i;

	}
	linesegment templine;
	int totalnum = 0;
	int totalnum2 = 0;
	int totalnum3 = 0;
	bool initial = 1;
	int newFaultNum = FaultNum;
	// in this program, radius is always a single elment array;
	radius[0] = r;
	countCenters[0] = 0;// record the number of centers;
	int iter = 0;
	initial = 1;
	singleloop = 0; // record how many interations required to cover the fault segments from the left to the right;
	Lastcircle = 0; // record the length covered by the last circle in each iteration;
					// for each loop, we need to cover the line segments as large as possible, then update the fractures and continue the loop;
	while (newFaultNum > 0)
	{
		countExtraFracture = 0;
		while (true)
		{
			// find the center for the next circle
			if (initial)// if this is the first fracture in each loop
			{
				Preindex[0].tempcenter.xcor = Fractures[0].xpo1;
				Preindex[0].tempcenter.ycor = Fractures[0].ypo1;
				Preindex[0].orignalFractureIndex = 0;
				FindNextCenter(Preindex, 1, radius[iter], xmin, xmax, ymin, ymax, n, iter);
			}
			else// if not the first circle, "Preindex" records the information of intersection points in the previous step
			{
				FindNextCenter(Preindex, totalnum3, radius[iter], xmin, xmax, ymin, ymax, n, iter);
			}
			initial = 0;
			// calculate that in the current circle, how many linesegments are covered. the top line segment should be included.
			Nblock = 0;
			tempcenter = centers[iter][countCenters[iter] - 1];
			//cout << "******************************************************************"<< endl;
			// check how many segments are coverd. 
			// Find the region for possible segments;
			templine.xpo1 = max(tempcenter.xcor - radius[iter], xmin);
			templine.ypo1 = max(tempcenter.ycor - radius[iter], ymin);
			templine.xpo2 = min(tempcenter.xcor + radius[iter], xmax);
			templine.ypo2 = min(tempcenter.ycor + radius[iter], ymax);
			BlockIndex(Allblocknumber, Nblock, n, templine);
			totalnum = 0;
			totalnum2 = 0;
			totalnum3 = 0;
			int numtemp = 0;
			for (int i = 0; i < Nblock; i++)
			{
				for (int j = 0; j < k[Fractureblock[i]]; j++)//check the intersections of fractures sharing the same block number
				{
					tempIntersectBlock[numtemp] = Allblocknumber[Fractureblock[i]][j];
					numtemp++;
				}
			}
			// remove duplicate fractures in each block;
			sort(tempIntersectBlock, tempIntersectBlock + numtemp);
			numtemp = RemoveDuplicate(tempIntersectBlock, numtemp);
			// check each possible segments to see whether it is covered by current circle or previous circles;
			for (int i = 0; i < numtemp; ++i)
			{
				double lengthx = Circle_Line_intersect(tempcenter, radius[iter], Fractures[tempIntersectBlock[i]], intersect1, intersect2);// caculate the length of intersecting part.
				if ((lengthx>0) && (segmentcovered[tempIntersectBlock[i]][0] == 0 || segmentcovered[tempIntersectBlock[i]][1] == 0))// they have an intersection and are not fully covered
				{
					if (check_line_circle_relation(tempcenter, radius[iter], Fractures[tempIntersectBlock[i]], intersect1, intersect2, tempIntersectBlock[i], totalnum2, iter, totalnum3))
					{
						totalnum++;
					}
				}
			}
			// If all the segments in the current circle are covered by current circle or previous circles, break this loop.
			if (totalnum == totalnum2)
			{
				sort(lastradius, lastradius + totalnum2, doublecompare);
				// the last circle should not be counted as a full circle, but a fraction of a circle
				Lastcircle = Lastcircle + lastradius[0] / (2.0*radius[iter]);// calculate the covered length in the last circle
				break;
			}

		}
		singleloop++;// calculate how many iterations we need to cover the line segments from left to right;
					 // update Fractures;
		FaultNum = newFaultNum;
		newFaultNum = 0;
		for (int i = 0; i < FaultNum; i++)
		{
			if (segmentcovered[i][0] == 0 || segmentcovered[i][1] == 0)
			{
				Fractures[newFaultNum] = Fractures[i];
				newFaultNum++;
			}
		}

		for (int i = 0; i < countExtraFracture; i++)
		{
			Fractures[newFaultNum] = ExtraFractures[i];
			newFaultNum++;
		}
		sort(Fractures, Fractures + newFaultNum, linecompare);// sort all fractures based on the y coordinates.

															  // reset the k and Allblocknumber to do the block calculation for the new loop
		reassign(10000, k);
		reassign(capacity, 2, segmentcovered);
		for (int i = 0; i < newFaultNum; i++)
		{
			Nblock = 0;
			Fracture = Fractures[i];
			BlockIndex(Allblocknumber, i, Nblock, n);
			tempFracturesindex[i] = i;

		}
		initial = 1;
	}

	// output the positions of alll circles;
	Outputcircles(iter, r, name);
	// update the number of circles, 
	circlenum = countCenters[iter] - singleloop + Lastcircle;
	return circlenum;
}

pointcor FractalNetwork::FindNextCenter(tempIndex*Preindex, int num, double r, double xmin, double xmax, double ymin, double ymax, double n, int iter)
{
	linesegment templine;
	sort(Preindex, Preindex + num, Indexcompare);// sort the intersections based on the vertical length;
	pointcor phighest = Preindex[num - 1].tempcenter;// the upper limit segments
	pointcor plowest = Preindex[0].tempcenter;// the lower limit segment
	int Nblock = 0;
	// search area for surrounding segments
	templine.xpo1 = phighest.xcor;
	templine.xpo2 = min(phighest.xcor + 2 * r, xmax);
	templine.ypo1 = max(phighest.ycor - sqrt(3)*r, ymin);
	templine.ypo2 = plowest.ycor;

	BlockIndex(Allblocknumber, Nblock, n, templine);
	countTempIndex = 0;

	int numtemp = 0;
	for (int i = 0; i < Nblock; i++)
	{

		for (int j = 0; j < k[Fractureblock[i]]; j++)//check the intersections of fractures sharing the same block number
		{
			tempIntersectBlock[numtemp] = Allblocknumber[Fractureblock[i]][j];
			numtemp++;
		}
	}
	tempIntersectBlock[numtemp] = Preindex[num - 1].orignalFractureIndex;// the upper line is included,
	numtemp++;
	// remove duplicate fractures in each block;
	sort(tempIntersectBlock, tempIntersectBlock + numtemp);
	numtemp = RemoveDuplicate(tempIntersectBlock, numtemp);
	// check the validation of the each segment wrt to the up-limit segment
	for (int i = 0; i < numtemp; ++i)
	{
		if (segmentcovered[tempIntersectBlock[i]][0] == 0 || segmentcovered[tempIntersectBlock[i]][1] == 0)
			checkvalid(phighest, Fractures[Preindex[num - 1].orignalFractureIndex], Fractures[tempIntersectBlock[i]], tempIntersectBlock[i], r);// check all possible lower limit of the circle coverage;
	}
	///////////////////////////////////////////////
	// if the number of surrounding segments is smaller than the previous intersection number (this is caused by non-parallel segments); 
	// we would break the lowest fracture and generate a new fracture
	if (countTempIndex < num)
	{
		segmentcovered[Preindex[0].orignalFractureIndex][1] = 1;
		ExtraFractures[countExtraFracture] = Fractures[Preindex[0].orignalFractureIndex];
		ExtraFractures[countExtraFracture].xpo1 = plowest.xcor;
		ExtraFractures[countExtraFracture].ypo1 = plowest.ycor;
		countExtraFracture++;
	}
	if (countTempIndex == 1) // no surrounding segments, just cover the only segment with a diameter.
	{
		pointcor dp = pol2cart(CalAngle(Fractures[Preindex[num - 1].orignalFractureIndex]), r);
		centers[iter][countCenters[iter]].xcor = phighest.xcor + dp.xcor;
		centers[iter][countCenters[iter]].ycor = phighest.ycor + dp.ycor;
		countCenters[iter]++;
	}
	else
	{
		// sort the intersecting linesegments based on its verticle distance to the highest line segment;
		sort(tempindex, tempindex + countTempIndex, Indexcompare);
		// if the number of surrounding segments equals the previous intersection number and the coverage is smaller than 2r (this is caused by non-parallel segments); 
		// we would break the lowest fracture and generate a new fracture
		if (countTempIndex == num &&  tempindex[0].templength < 2 * r)
		{
			// set the second point of the lowest fracture as visited; 
			segmentcovered[Preindex[0].orignalFractureIndex][1] = 1;
			ExtraFractures[countExtraFracture] = Fractures[Preindex[0].orignalFractureIndex];
			ExtraFractures[countExtraFracture].xpo1 = plowest.xcor;
			ExtraFractures[countExtraFracture].ypo1 = plowest.ycor;
			countExtraFracture++;
		}
		for (int i = 0; i < countTempIndex; i++)// transverse all possible fault segments and find the farthest and coverage larger than the diameter fault segment;
		{
			if (tempindex[i].templength >= 2 * r)// the secent line has the length larger than the diameter;
			{
				if (tempindex[i].orignalFractureIndex == Preindex[0].orignalFractureIndex)// if there is no lower line segment below the plowest line segment.
				{

					if ((phighest.ycor - plowest.ycor) / (phighest.xcor - plowest.xcor) < 0 || Iszero(phighest.xcor - plowest.xcor))
					{
						centers[iter][countCenters[iter]] = tempindex[i].tempcenter;
						countCenters[iter]++;
					}
					else// if the slope is positive, we will just use the secent line to find the new center;
					{
						pointcor intersect1, intersect2;
						Circle_Line_intersect(plowest, 2 * r, Fractures[Preindex[num - 1].orignalFractureIndex], intersect1, intersect2);// pay attention, here is 2*r.
						centers[iter][countCenters[iter]].xcor = (plowest.xcor + intersect2.xcor) / 2.0;
						centers[iter][countCenters[iter]].ycor = (plowest.ycor + intersect2.ycor) / 2.0;
						countCenters[iter]++;
					}
				}
				else
				{
					if ((phighest.ycor - plowest.ycor) / (phighest.xcor - plowest.xcor) < 0 || Iszero(phighest.xcor - plowest.xcor))
					{
						centers[iter][countCenters[iter]] = tempindex[i].tempcenter;
						countCenters[iter]++;
					}
					else// if the slope is positive, we will just use the secent line to find the new center;
					{
						double alpha = CalAngle(phighest, plowest);
						pointcor middle;
						middle.xcor = (phighest.xcor + plowest.xcor) / 2.0;
						middle.ycor = (phighest.ycor + plowest.ycor) / 2.0;
						double dis1 = LengthCal(phighest, plowest) / 2.0;
						double rr = sqrt(r*r - dis1*dis1);
						pointcor dp = pol2cart(alpha - PI / 2, rr);
						centers[iter][countCenters[iter]].xcor = middle.xcor + dp.xcor;
						centers[iter][countCenters[iter]].ycor = middle.ycor + dp.ycor;
						countCenters[iter]++;
					}
				}

				break;
			}
		}

	}

	return centers[iter][countCenters[iter] - 1];
}

bool FractalNetwork::FractalNetwork::AllpreviousIncluded(pointcor center, double r, tempIndex previous, int num)
{
	int count = 0;
	for (int i = 0; i < num; i++)
	{
		if (pointIncircle(center, r, previous.tempcenter))
			count++;
	}
	if (count == num)
		return 1;
	else
		return 0;
}

bool FractalNetwork::check_line_circle_relation(pointcor center, double r, linesegment line, pointcor intersect1, pointcor intersect2, int Fnum, int &totalnum, int iter, int &totalnum3)
{
	// check whether line is covered by the current circle or previous circles;
	pointcor p1, p2;
	p1.xcor = line.xpo1; p1.ycor = line.ypo1;
	p2.xcor = line.xpo2; p2.ycor = line.ypo2;
	pointcor tempcenter = center;
	if (CheckPointInsideLineSegement(intersect1, line) || CheckPointInsideLineSegement(intersect2, line) || pointIncircle(center,r,p1) || pointIncircle(center,r,p2))// if at least one interesction point is in the line;
	{
		if (pointIncircle(tempcenter, r, p1) && (segmentcovered[Fnum][0] == 0))
		{
			segmentcovered[Fnum][0] = 1;

		}
		else if (!pointIncircle(tempcenter,r, p1) && (segmentcovered[Fnum][0] == 0))
		{// If P1 is not covered by current or previous circles, we should divide the current segment into two parts,and left part should be a new segment and be covered in next iteration.
			segmentcovered[Fnum][0] = 1;
			ExtraFractures[countExtraFracture] = line;
			ExtraFractures[countExtraFracture].xpo2 = intersect1.xcor;
			ExtraFractures[countExtraFracture].ypo2 = intersect1.ycor;
			countExtraFracture++;
		}
		 
		if (pointIncircle(tempcenter,r, p2) && (segmentcovered[Fnum][1] == 0))
		{
			segmentcovered[Fnum][1] = 1;
		}
		if (!pointIncircle(tempcenter,r, p2))
		{
			// if the right point of the segment is not covered, record the right intersection point in Preindex;
			Preindex[totalnum3].orignalFractureIndex = Fnum;
			Preindex[totalnum3].line = line;
			Preindex[totalnum3].tempcenter = intersect2;// record the intersection point.
			Preindex[totalnum3].vertdis = -intersect2.ycor;
			totalnum3++;
		}
		if ((segmentcovered[Fnum][1] == 1) && (segmentcovered[Fnum][0] == 1))// the line segment is fully covered;
		{
			// the segment may not span the current circle, we will use a fraction instead of 1 to avoid overestimation;
			lastradius[totalnum] = LengthCal(intersect1, p2);
			totalnum++;
		}
		 
		return 1;
	}
	return 0;
}

bool FractalNetwork::CheckPointInsideLineSegement(pointcor point, linesegment line)
{
	double length = LengthCal(line);
	double length1 = sqrt((line.xpo1 - point.xcor)*(line.xpo1 - point.xcor) + (line.ypo1 - point.ycor)*(line.ypo1 - point.ycor));
	double length2 = sqrt((line.xpo2 - point.xcor)*(line.xpo2 - point.xcor) + (line.ypo2 - point.ycor)*(line.ypo2 - point.ycor));
	double diff = length - length1 - length2;
	if (Iszero(diff))
		return true;
	else
		return false;

}
//**********************************************************************************************
double FractalNetwork::LengthCal(linesegment linex)
{
	double mx = linex.xpo2 - linex.xpo1;
	double my = linex.ypo2 - linex.ypo1;
	double length = sqrt(mx*mx + my * my);
	return length;

}

void FractalNetwork::checkvalid(pointcor newpoint, linesegment line1, linesegment line2, int Fnum, double r)
{
	// this is function is used to check whether the given linesegment line2 intersect the large circle whose center is the phighest point and radii is 2r.
	pointcor tempcenter; double vertdis;
	tempcenter = newpoint;
	pointcor tempintersect1, tempintersect2;
	double tmeplength;
	double totalintersectlength;
	double tempr = 2 * r;
	pointcor p1, p2, p3; p1=newpoint;
	p2.xcor = line2.xpo1; p2.ycor = line2.ypo1;
	p3.xcor = line2.xpo2; p3.ycor = line2.ypo2;
	Circle_Line_intersect(tempcenter, tempr, line2, tempintersect1, tempintersect2);
 
 	if (tempintersect1.xcor != NN)// there is a intersection point;
	{
		// at least one intersection point is in the line segment or the line segment is in the circle
		if (CheckPointInsideLineSegement(tempintersect1, line2) || CheckPointInsideLineSegement(tempintersect2, line2) || pointIncircle(tempcenter, tempr, p2) || pointIncircle(tempcenter, tempr, p3))
		{
			tempcenter.xcor = (p1.xcor + tempintersect2.xcor) / 2.0;
			tempcenter.ycor = (p1.ycor + tempintersect2.ycor) / 2.0;
			tempr = LengthCal(p1, tempintersect2) / 2.0;
			vertdis = dotTolineDis(tempintersect2, line1);
			tempindex[countTempIndex].orignalFractureIndex = Fnum;
			tempindex[countTempIndex].tempcenter = tempcenter;
			tempindex[countTempIndex].templength = secentlength(tempcenter, r, line1) + secentlength(tempcenter, r, line2);// if the summation of the secent lengths are larger than 2R, we can use one circle cover the maximum number of segments;
			tempindex[countTempIndex].vertdis = vertdis;
			tempindex[countTempIndex].line = line2;
			countTempIndex++;
		}
	}

}

double FractalNetwork::secentlength(pointcor center, double r, linesegment line)
{
	double h = dotTolineDis(center, line);
	double result = 2 * sqrt(r*r - h*h);
	return result;
}
 
double FractalNetwork::min(double x, double y)
{
if (x > y)
return y;
return x;
}

double FractalNetwork::max(double x, double y)
{
if (x > y)
return x;
return y;
}

double FractalNetwork::Circle_Line_intersect(pointcor center, double r, linesegment line, pointcor& intersect1, pointcor& intersect2)
{
	// this function is used to find the intersection points between a line segment and circle;
	// return the length of the intersecting part.
	// https://brilliant.org/wiki/dot-product-distance-between-point-and-a-line/
	double length;
	intersect1.xcor = NN; intersect1.ycor = NN; intersect2 = intersect1; length = 0;
	double dis = dotTolineDis(center, line);
	linesegment inside;
	pointcor p1, p2; p1.xcor = line.xpo1; p1.ycor = line.ypo1; p2.xcor = line.xpo2; p2.ycor = line.ypo2;
	if (dis > r)// no intersection points
	{
		length = 0;
	}
	else if (dis < r)
	{
		double k = (line.ypo1 - line.ypo2) / (line.xpo1 - line.xpo2);
		double b = line.ypo1 - k*line.xpo1;
		double A = (1 + k*k);
		double B = 2 * k*(b - center.ycor) - 2 * center.xcor;
		double C = center.xcor*center.xcor + b*b + center.ycor*center.ycor - 2 * b*center.ycor - r*r;
		pointcor intersectX = SolveQua(A, B, C);// solve for the intersection  points , the x coordinates
		intersect1.xcor = intersectX.xcor; intersect1.ycor = k*intersect1.xcor + b;
		intersect2.xcor = intersectX.ycor; intersect2.ycor = k*intersect2.xcor + b;
		pointcor temp;
		temp = intersect2;
		if (intersect1.xcor > intersect2.xcor)// make sure the intersect1 has the smaller x coordinates.
		{
			intersect2 = intersect1;
			intersect1 = temp;

		}
		if (pointIncircle(center, r, p1) && (pointIncircle(center, r, p2)))
		{
			inside = line;
			length = LengthCal(inside);
		}
		else if (pointIncircle(center, r, p1))
		{
			inside.xpo1 = p1.xcor; inside.ypo1 = p1.ycor;
			inside.xpo2 = intersect2.xcor; inside.ypo2 = intersect2.ycor;
			length = LengthCal(inside);
		}
		else if (pointIncircle(center, r, p2))
		{
			inside.xpo1 = p2.xcor; inside.ypo1 = p2.ycor;
			inside.xpo2 = intersect1.xcor; inside.ypo2 = intersect1.ycor;
			length = LengthCal(inside);
		}
		else if(!pointIncircle(center, r, p1) && (!pointIncircle(center, r, p2)) && (center.xcor<=p2.xcor) && (center.xcor>=p1.xcor))// the line is outside the circle;
		{
			inside.xpo1 = intersect2.xcor; inside.ypo1 = intersect2.ycor;
			inside.xpo2 = intersect1.xcor; inside.ypo2 = intersect1.ycor;
			length = LengthCal(inside);
		 
		}
		else
		{
			length = 0;
		}
	}
	else // tangent line of the circle
	{
		length = 0;
		intersect1.xcor = NN; intersect1.ycor = NN; intersect2 = intersect1;
	}

	
	return length;
}

double FractalNetwork::dotTolineDis(pointcor center, linesegment line)
{
	

	double a = (line.ypo1 - line.ypo2) / (line.xpo1 - line.xpo2);
	double c = line.ypo1 - a*line.xpo1;
	double b = -1;
	double d = abs(a*center.xcor + b*center.ycor + c) / (sqrt(a*a + b*b));
	return d;

 
}

double FractalNetwork::dotproduct(pointcor p1, pointcor p2)
{
	double result;
	result = p1.xcor*p2.xcor + p1.ycor*p2.ycor;
	return result;
}

bool FractalNetwork::pointIncircle(pointcor center, double r, pointcor p)
{
	double dis = LengthCal(p, center);
	if (Iszero(dis -r) || dis<r)
		return 1;
	else
		return 0;
}

void FractalNetwork::reassign(int num1, int num2, bool**array1)
{


	for (int i = 0; i < num1; i++)
	{

		for (int j = 0; j < num2; j++)
			array1[i][j] = 0;
	}
}

void FractalNetwork::reassign(int num1, int*array1)
{

	for (int i = 0; i < num1; i++)

		array1[i] = 0;
}

pointcor FractalNetwork::SolveQua(double A, double B, double C)
{
	// solve the quadratic function and find the roots
	pointcor result; result.xcor = NN; result.ycor = NN;
	double delta = B * B - 4 * A*C;
	if (delta < 0)
		return result;
	else
	{
		result.ycor = (-B + sqrt(delta)) / (2 * A);
		result.xcor = (-B - sqrt(delta)) / (2 * A);
		return result;
	}
}


//*********************************  Block Calculation Functions *******************************************
void FractalNetwork::BlockIndex(int** Allblocknumber, int FractureNum, int &Nblock, int n)
{
	int rownum = L / bs;
	double xmin, xmax;
	if (Fracture.xpo1 >= Fracture.xpo2)
	{
		xmin = Fracture.xpo2;
		xmax = Fracture.xpo1;
	}
	else
	{
		xmin = Fracture.xpo1;
		xmax = Fracture.xpo2;
	}
	double ymin, ymax;
	if (Fracture.ypo1 >= Fracture.ypo2)
	{
		ymin = Fracture.ypo2;
		ymax = Fracture.ypo1;
	}
	else
	{
		ymin = Fracture.ypo1;
		ymax = Fracture.ypo2;
	}
	int Xmin = Findlastmin(xcord, xmin, n);
	int Xmax = Findfirstmax(xcord, xmax, n);
	int Ymin = Findlastmin(ycord, ymin, n);
	int Ymax = Findfirstmax(ycord, ymax, n);
	int X1 = Xmin / bs;
	int X2 = Xmax / bs;
	int Y1 = Ymin / bs;
	int Y2 = Ymax / bs;
	if (Y2 == Y1 + 1) //if the fractures are in the same row
	{

		for (int j = 0; j < (X2 - X1); j++)
		{
			Allblocknumber[Y1*rownum + X1 + j][k[Y1*rownum + X1 + j]] = FractureNum;
			Fractureblock[Nblock] = (Y1*rownum + X1 + j);
			Nblock = Nblock + 1;
			k[Y1*rownum + X1 + j] = k[Y1*rownum + X1 + j] + 1;
		}
	}
	else if (Y2>Y1 + 1)
	{
		for (int j2 = 0; j2<(Y2 - Y1); j2++)
		{
			for (int i = 0; i < (X2 - X1); i++)
			{
				Allblocknumber[(Y1 + j2)*rownum + X1 + i][k[(Y1 + j2)*rownum + X1 + i]] = FractureNum;
				Fractureblock[Nblock] = (Y1 + j2)*rownum + X1 + i;
				Nblock = Nblock + 1;
				k[(Y1 + j2)*rownum + X1 + i] = k[(Y1 + j2)*rownum + X1 + i] + 1;

			}

		}
	}
	else
	{
		system("pause");
		cout << "I know the error" << endl;
	}



}
//*******************************************************************************************************************************//
void FractalNetwork::BlockIndex(int** Allblocknumber, int &Nblock, int n, linesegment Fracture)
{
	int rownum = L / bs;
	double xmin, xmax;
	if (Fracture.xpo1 >= Fracture.xpo2)
	{
		xmin = Fracture.xpo2;
		xmax = Fracture.xpo1;
	}
	else
	{
		xmin = Fracture.xpo1;
		xmax = Fracture.xpo2;
	}
	double ymin, ymax;
	if (Fracture.ypo1 >= Fracture.ypo2)
	{
		ymin = Fracture.ypo2;
		ymax = Fracture.ypo1;
	}
	else
	{
		ymin = Fracture.ypo1;
		ymax = Fracture.ypo2;
	}
	int Xmin = Findlastmin(xcord, xmin, n);
	int Xmax = Findfirstmax(xcord, xmax, n);
	int Ymin = Findlastmin(ycord, ymin, n);
	int Ymax = Findfirstmax(ycord, ymax, n);
	int X1 = Xmin / bs;
	int X2 = Xmax / bs;
	int Y1 = Ymin / bs;
	int Y2 = Ymax / bs;
	if (Y2 == Y1 + 1) //if the fractures are in the same row
	{

		for (int j = 0; j < (X2 - X1); j++)
		{

			Fractureblock[Nblock] = (Y1*rownum + X1 + j);
			Nblock = Nblock + 1;

		}
	}
	else if (Y2>Y1 + 1)
	{
		for (int j2 = 0; j2<(Y2 - Y1); j2++)
		{
			for (int i = 0; i < (X2 - X1); i++)
			{
				Fractureblock[Nblock] = (Y1 + j2)*rownum + X1 + i;
				Nblock = Nblock + 1;


			}

		}
	}
	else
	{
		system("pause");
		cout << "I know the error" << endl;
	}
}

//********************************  Assignment Function  *****************************************

void FractalNetwork::AssignCapacity(int outtercapacity, int outterL, int outterbs)
{
	capacity = outtercapacity;
	L = outterL;
	bs = outterbs;
	int rowNum = 100;
	//allocate space for each container;
	Allblocknumber = new int*[rowNum*rowNum];
	k = new int[rowNum*rowNum];//this is used to record every block has how many fracturs;

	for (int i = 0; i < rowNum*rowNum; i++)
	{
		Allblocknumber[i] = new int[capacity];
		k[i] = 0;
	}
	tempIntersectBlock = new int[capacity];
	Fractureblock = new int[capacity];
	// generate the xcord and ycord for the blocks;
	xcord = new int[L / bs + 1];
	ycord = new int[L / bs + 1];
	Length = new double[capacity];
	Fractures = new linesegment[capacity];
	tempFracturesindex = new int[capacity];
	centers = new pointcor*[100];//record the centers of circles under different radius;
	radius = new double[1000];
	for (int i = 0; i < 100; i++)
	{
		centers[i] = new pointcor[100000];
	}
	tempindex = new tempIndex[1000]; countTempIndex = 0;
	segmentcovered = new bool*[capacity];
	ExtraFractures = new linesegment[capacity];
	countCenters = new int[capacity];
	for (int i = 0; i < capacity; i++)
	{
		countCenters[i] = 0;
		segmentcovered[i] = new bool[2];
		for (int j = 0; j < 2; j++)
			segmentcovered[i][j] = 0;
	}
	Preindex = new tempIndex[1000];
	lastradius = new double[10000];
	totalfractures = new linesegment[capacity];
}

//********************************* Geometry Manipulation Functions ********************************

//***************************************************************
int FractalNetwork::Generatexycord(int L, int bs, int * xcord, int *ycord) {
	int n = 0;
	for (int i1 = 0; i1 < L + 1; i1 = i1 + bs)
	{

		xcord[n] = i1;

		ycord[n] = i1;
		n++;
	}
	return n;
}
//***************************************************************
int FractalNetwork::Generatexycord(int L, int bs, int * xcord, int *ycord, int xmin, int ymin)
{
	int n = 0;
	for (int i1 = 0; i1 < L + bs; i1 = i1 + bs)
	{

		xcord[n] = xmin + i1;

		ycord[n] = ymin + i1;

		n++;
	}

	return n;
}
//***************************************************************


//***************************************************************
pointcor   FractalNetwork::pol2cart(double th, double r)
{
	pointcor re;
	re.xcor = r*cos(th);
	re.ycor = r*sin(th);
	return re;
}

//***************************************************************
bool FractalNetwork::Iszero(double d)
{
	return abs(d) < tol;
}

bool FractalNetwork::Iszero(int d)
{
	return abs(d) < tol;
}
double FractalNetwork::LengthCal(pointcor p1, pointcor p2)
{
	double mx = p1.xcor - p2.xcor;
	double my = p1.ycor - p2.ycor;
	double length = sqrt(mx*mx + my*my);
	return length;

}

//find the supremum function （the smallest upper bound), this function is order of log(N)
int FractalNetwork::Findfirstmax(int *vec, double x, int n) {
	return BinaryFindfirstmax(x, vec, 0, n - 1);
}
int FractalNetwork::BinaryFindfirstmax(double x, int *vec, int p1, int p2) {

	if (p2 - p1 == 1) return vec[p2];
	int mid = (p1 + p2) / 2;
	if (x < vec[mid]) {
		return BinaryFindfirstmax(x, vec, p1, mid);
	}
	else {
		return BinaryFindfirstmax(x, vec, mid, p2);
	}

}

//***************************************************************
//find the infimum function （the largest lower bound）
int FractalNetwork::Findlastmin(int *vec, double x, int n) {
	return BinaryFindlastmin(x, vec, 0, n - 1);
}
int FractalNetwork::BinaryFindlastmin(double x, int *vec, int p1, int p2) {

	if (p2 - p1 == 1) return vec[p1];
	int mid = (p1 + p2) / 2;
	if (x < vec[mid]) {
		return BinaryFindlastmin(x, vec, p1, mid);
	}
	else {
		return BinaryFindlastmin(x, vec, mid, p2);
	}

}

int FractalNetwork::RemoveDuplicate(linesegment a[], int size)
{
	int prev = 0;
	for (int i = 0; i < size; ++i) {
		if (a[i].xpo1 != a[prev].xpo1 || a[i].ypo1 != a[prev].ypo1 || a[i].xpo2 != a[prev].xpo2 || a[i].ypo2 != a[prev].ypo2)
		{
			a[prev + 1].xpo1 = a[i].xpo1;
			a[prev + 1].xpo2 = a[i].xpo2;
			a[prev + 1].ypo2 = a[i].ypo2;
			a[prev + 1].ypo1 = a[i].ypo1;
			prev++;
		}

	}
	int count = prev + 1;
	return count;
}

double FractalNetwork::CalAngle(linesegment line)
{
	double result;
	result = atan((line.ypo2 - line.ypo1) / (line.xpo2 - line.xpo1));
	return result;
}

double FractalNetwork::CalAngle(pointcor p1, pointcor p2)
{
	double result;
	result = atan((p2.ycor - p1.ycor) / (p2.xcor - p1.xcor));
	return result;
}

void FractalNetwork::Translate2D(linesegment &line, double px, double py)
{
	pointcor p1, p2;
	p1.xcor = line.xpo1; p2.xcor = line.xpo2;
	p1.ycor = line.ypo1; p2.ycor = line.ypo2;
	p1 = Translate2D(p1, px, py);
	p2 = Translate2D(p2, px, py);
	line.xpo1 = p1.xcor; line.ypo1 = p1.ycor;
	line.xpo2 = p2.xcor; line.ypo2 = p2.ycor;

}

void FractalNetwork::transformFractures(int num, double angle, double &xmax, double &xmin, double &ymax, double &ymin)
{

	linesegment temp;
	pointcor center;
	xmin = NN;  ymax = -NN;  xmax = -NN;  ymin = NN;
	for (int i = 0; i < num; i++)
	{
		//center = FindCenter(Fractures[i]);
		center.xcor = 0; center.ycor = 0;
		rotate2D(Fractures[i], -angle, center);// make all fractures parallel to the x axis,to make the calcualtion simpler.
											   //rotate2D(Fractures[i], -CalAngle(Fractures[i]), center);
		if (Fractures[i].xpo1 < Fractures[i].xpo2)
		{
			if (Fractures[i].xpo1 < xmin)
				xmin = Fractures[i].xpo1;
			if (Fractures[i].xpo2 > xmax)
				xmax = Fractures[i].xpo2;
		}
		else
		{
			if (Fractures[i].xpo2 < xmin)
				xmin = Fractures[i].xpo2;
			if (Fractures[i].xpo1 > xmax)
				xmax = Fractures[i].xpo1;
			temp = Fractures[i];// make fractures sorted by the x coordinates
			Fractures[i].xpo1 = temp.xpo2; Fractures[i].ypo1 = temp.ypo2;
			Fractures[i].xpo2 = temp.xpo1; Fractures[i].ypo2 = temp.ypo1;

		}
		////////////////////
		if (Fractures[i].ypo1 < Fractures[i].ypo2)
		{
			if (Fractures[i].ypo2 > ymax)
				ymax = Fractures[i].ypo2;
			if (Fractures[i].ypo1 < ymin)
				ymin = Fractures[i].ypo1;
		}
		else
		{
			if (Fractures[i].ypo2 < ymin)
				ymin = Fractures[i].ypo2;
			if (Fractures[i].ypo1 > ymax)
				ymax = Fractures[i].ypo1;
		}
	}
}

int FractalNetwork::RemoveDuplicate(int a[], int size)
{
	int prev = 0;
	for (int i = 0; i < size; ++i) {
		if (a[i] != a[prev])
			a[++prev] = a[i];
	}
	int count = prev + 1;
	return count;
}

void FractalNetwork::rotate2D(linesegment &line, double theta, pointcor centerpoint) {
	double sinttheta = sin(theta);
	double costheta = cos(theta);
	pointcor p1, p2;
	p1.xcor = line.xpo1; p2.xcor = line.xpo2;
	p1.ycor = line.ypo1; p2.ycor = line.ypo2;
	rotate2D(p1, sinttheta, costheta, centerpoint);
	rotate2D(p2, sinttheta, costheta, centerpoint);
	line.xpo1 = p1.xcor; line.ypo1 = p1.ycor;
	line.xpo2 = p2.xcor; line.ypo2 = p2.ycor;
}

void FractalNetwork::rotate2D(pointcor &point, double sintheta, double costheta, pointcor centerpoint)
{
	pointcor point1; point1 = point;
	point.xcor = (point1.xcor - centerpoint.xcor)*costheta - (point1.ycor - centerpoint.ycor)*sintheta + centerpoint.xcor;
	point.ycor = (point1.xcor - centerpoint.xcor)*sintheta + (point1.ycor - centerpoint.ycor)*costheta + centerpoint.ycor;

}

linesegment FractalNetwork::Translate2D(linesegment line, double px, double py, double n)
{
	linesegment result;
	pointcor p1, p2;
	p1.xcor = line.xpo1; p2.xcor = line.xpo2;
	p1.ycor = line.ypo1; p2.ycor = line.ypo2;
	p1 = Translate2D(p1, px, py);
	p2 = Translate2D(p2, px, py);
	result.xpo1 = p1.xcor; result.ypo1 = p1.ycor;
	result.xpo2 = p2.xcor; result.ypo2 = p2.ycor;
	return result;

}

pointcor FractalNetwork::Translate2D(pointcor &point, double px, double py)
{
	pointcor pointnew;
	pointnew.xcor = point.xcor + px;
	pointnew.ycor = point.ycor + py;
	point = pointnew;
	return pointnew;
}


//************************************************ Input Functions  ***************************************************

int FractalNetwork::InputFracture(string filename, double &xmax, double &xmin, double &ymax, double &ymin, int &largestlengthindex, double &totallength)
{
	stringstream  filename1;

	filename1 << "Data/" << filename << ".txt";
	ifstream inputdata;
	inputdata.open(filename1.str());
	int i = 0;
	linesegment temp;
	double length = 0;
	double templength;

	while (!inputdata.eof())
	{// input the fracture;
		inputdata >> Fractures[i].xpo1 >> Fractures[i].ypo1 >> Fractures[i].xpo2 >> Fractures[i].ypo2;
		//  calculate the segment lengths and find out the index of the segment with the largest length.
		// the segments set will be rotated based on the angle of the segment with the largest length;
		templength = LengthCal(Fractures[i]);
		totallength = totallength + templength;
		if (templength > length)
		{
			length = templength;
			largestlengthindex = i;
		}
		// rotate all linesegments and make the be parallel with the x axis (for generated segments)
		// pointcor center = FindCenter(Fractures[i]);
		// rotate2D(Fractures[i], -CalAngle(Fractures[i]), center);// make all fractures parallel to the x axis,to make the calcualtion simpler.
		// find the xmin, ymin, xmax and ymax value.
		if (Fractures[i].xpo1 < Fractures[i].xpo2)
		{
			if (Fractures[i].xpo1 < xmin)
				xmin = Fractures[i].xpo1;
			if (Fractures[i].xpo2 > xmax)
				xmax = Fractures[i].xpo2;
		}
		else
		{
			if (Fractures[i].xpo2 < xmin)
				xmin = Fractures[i].xpo2;
			if (Fractures[i].xpo1 > xmax)
				xmax = Fractures[i].xpo1;
			temp = Fractures[i];// make fractures sorted by the x coordinates
			Fractures[i].xpo1 = temp.xpo2; Fractures[i].ypo1 = temp.ypo2;
			Fractures[i].xpo2 = temp.xpo1; Fractures[i].ypo2 = temp.ypo1;

		}


		if (Fractures[i].ypo1 < Fractures[i].ypo2)
		{
			if (Fractures[i].ypo2 > ymax)
				ymax = Fractures[i].ypo2;
			if (Fractures[i].ypo1 < ymin)
				ymin = Fractures[i].ypo1;
		}
		else
		{
			if (Fractures[i].ypo2 < ymin)
				ymin = Fractures[i].ypo2;
			if (Fractures[i].ypo1 > ymax)
				ymax = Fractures[i].ypo1;
		}
		i++;
	}
	inputdata.close();
	return i;
}

int FractalNetwork::InputFracture(string filename, double &totallength)
{
	stringstream  filename1;
	filename1 << filename << ".txt";
	ifstream inputdata;
	inputdata.open(filename1.str());
	Fractures = new linesegment[1000];
	int i = 0;
	while (!inputdata.eof())
	{// input the fracture;
		inputdata >> Fractures[i].xpo1 >> Fractures[i].ypo1 >> Fractures[i].xpo2 >> Fractures[i].ypo2;
		totallength = totallength + LengthCal(Fractures[i]);
		i++;
	}
	inputdata.close();
	return i;//  Usually there is a blank line at last, so we need to minus 1 to remove that line.
}

int FractalNetwork::InputCenter(string filename, double r, pointcor* centers)
{
	stringstream  filename1;
	filename1 << filename << "_circle_" << r << ".txt";
	cout << filename1.str() << endl;
	//system("pause");
	ifstream inputdata;
	inputdata.open(filename1.str());
	int i = 0;
	while (!inputdata.eof())
	{
		inputdata >> centers[i].xcor >> centers[i].ycor;
		i++;
	}
	inputdata.close();
	return i - 1;//  Usually there is a blank line at last, so we need to minus 1 to remove that line.
}


void FractalNetwork::Inputstring(string& name)
{
	int input;
	cout << "*************************************************************" << endl;
	cout << "Please choose a Fault for the calculation" << endl;
	cout << "*************************************************************" << endl;
	cout << "You can find the data in the folder 'Data'" << endl;
	cout << "*************************************************************" << endl;
	cout << " You can choose from:" << endl;
	cout << "1: RealFault1 (San Andreas Fault System)" << endl;
	cout << "2: RealFault2 (Quaternary Fault array in the central Apennines in Italy)" << endl;
	cout << "3: RealFault3 (Top Kharaib Fracture Lineaments from Lekhwair Field)" << endl;
	cout << "4: generated1 (generated fault segments with mean segment length equal to 80 m)" << endl;
	cout << "5: generated2 (generated fault segments with mean segment length equal to 240 m)" << endl;
	cout << "6: generated3 (generated fault segments with mean segment length equal to 800 m)" << endl;
	cout << "*************************************************************" << endl;
	cout << "Please type your choice" << endl;
	cout << "*************************************************************" << endl;
	cin >> input;
	if (input == 1)
	{
		name = "RealFault1";
		cout << "Your choice is San Andreas Fault System" << endl;
	}
	else if (input == 2)
	{
		name = "RealFault2";
		cout << "Your choice is Quaternary Fault array in the central Apennines in Italy" << endl;
	}
	else if (input == 3)
	{
		name = "RealFault3";
		cout << "Your choice is Top Kharaib Fracture Lineaments from Lekhwair Field" << endl;
	}
	else if (input == 4)
	{
		name = "generated1";
		cout << "Your choice is the generated fault segments with mean segment length equal to 80m" << endl;
	}
	else if (input == 5)
	{
		name = "generated2";
		cout << "Your choice is the generated fault segments with mean segment length equal to 240m" << endl;
	}
	else if (input == 6)
	{
		name = "generated3";
		cout << "Your choice is the generated fault segments with mean segment length equal to 800m" << endl;
	}
	else
	{
	 
		cout << "You did not choose the right number. Please choose the right number for a fault map" << endl;
		exit(EXIT_FAILURE);
	}

	cout << "**************************************************************" << endl;
	cout << "All calculation results can be found in the folder 'Data/Results'" << endl;

}
//******************************************* Output Functions  *******************************************
void  FractalNetwork::OutputFracture(string name, int num, linesegment* fractures)
{
	ofstream output;
	if (output.fail())
	{
		cout << "openning output 1 file failed" << endl;
		system("pause");
		exit(1);
	}
	output.open(name);
	output.setf(ios::showpoint);
	output.precision(8);

	for (int i = 0; i < num; i++)
	{

		output << fractures[i].xpo1 << "  " << fractures[i].ypo1 << "  " << fractures[i].xpo2 << "  " << fractures[i].ypo2 << endl;


	}
	output.close();

}

void  FractalNetwork::Outputarray(string name, int num, double *array)
{

	ofstream output1;
	output1.open(name);
	output1.setf(ios::showpoint);
	output1.precision(8);
	for (int i = 0; i < num; i++)
	{
		output1 << array[i] << endl;
	}

}

void  FractalNetwork::Outputcircles(int num, double r, string name)
{
	ofstream output;
	stringstream  filename1;
	filename1 << "Data/Results/" << name << "_circle_" << r << ".txt";
	output.open(filename1.str());
	output.setf(ios::showpoint);
	output.precision(8);
	for (int j = 0; j < countCenters[num]; j++)
	{
		output << centers[num][j].xcor << "  " << centers[num][j].ycor << endl;
	}
}







